/*
 * $Id: MemoryTTree.java,v 1.1 2005/06/30 01:14:44 ahimanikya Exp $
 * =======================================================================
 * Copyright (c) 2005 Axion Development Team.  All rights reserved.
 *  
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions 
 * are met:
 * 
 * 1. Redistributions of source code must retain the above 
 *    copyright notice, this list of conditions and the following 
 *    disclaimer. 
 *   
 * 2. Redistributions in binary form must reproduce the above copyright 
 *    notice, this list of conditions and the following disclaimer in 
 *    the documentation and/or other materials provided with the 
 *    distribution. 
 *   
 * 3. The names "Tigris", "Axion", nor the names of its contributors may 
 *    not be used to endorse or promote products derived from this 
 *    software without specific prior written permission. 
 *  
 * 4. Products derived from this software may not be called "Axion", nor 
 *    may "Tigris" or "Axion" appear in their names without specific prior
 *    written permission.
 *   
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY 
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * =======================================================================
 */

package org.axiondb.ext.indexes.ttree;

import java.util.Comparator;

import org.axiondb.AxionException;

/**
 * Memory T-Tree implemenation.
 * 
 * @version $Revision: 1.1 $ $Date: 2005/06/30 01:14:44 $
 * @author Pavels Andrejevs
 */
public class MemoryTTree extends TTree {


    public class TTreeNode extends AbstractTTreeNode {

        private Object[] _keys;
        private int[] _values;

        public TTreeNode(TTree tree) throws AxionException {
            this(tree, 0);
        }

        public TTreeNode(TTree tree, int fileId) throws AxionException {
            super(tree, fileId);
        }

        public void addKeyValue(Object key, int value) throws AxionException {
            setKey(_nodeSize, key);
            setValue(_nodeSize, value);
            _nodeSize++;
        }

        public void copyKeysValues(int from, int to, int length) throws AxionException {
            copyKeys(from, to, length);
            copyValues(from, to, length);
        }

        public void insertKeyValue(int index, Object key, int value) throws AxionException {
            insertKey(index, key);
            insertValue(index, value);
            _nodeSize++;
        }

        public void removeKeyValue(int index) throws AxionException {
            removeKey(index);
            removeValue(index);
            _nodeSize--;
        }

        public void setKeyValue(int index, Object key, int value) throws AxionException {
            setKey(index, key);
            setValue(index, value);
        }

        protected void copyKeys(int from, int to, int length) throws AxionException {
            System.arraycopy(_keys, from, _keys, to, length);
        }

        protected final void copyValues(int from, int to, int length) throws AxionException {
            System.arraycopy(_values, from, _values, to, length);
        }

        protected Object[] createKeys() {
            if (_keys == null) {
                _keys = new Object[_tree.getMaxNodeSize()];
            }
            return _keys;
        }

        protected final int[] createValues() {
            if (_values == null) {
                _values = new int[_tree.getMaxNodeSize()];
            }
            return _values;
        }

        protected void destroyKeys() {
            _keys = null;
        }

        protected final void destroyValues() {
            _values = null;
        }

        protected Object getKey(int index) throws AxionException {
            return _keys[index];
        }

        protected Object[] getKeys() throws AxionException {
            return _keys;
        }

        protected final int getValue(int index) throws AxionException {
            return _values[index];
        }

        protected final int[] getValues() throws AxionException {
            return _values;
        }

        protected void insertKey(int index, Object key) throws AxionException {
            System.arraycopy(_keys, index, _keys, index + 1, _nodeSize - index);
            _keys[index] = key;
        }

        protected final void insertValue(int index, int value) throws AxionException {
            System.arraycopy(_values, index, _values, index + 1, _nodeSize - index);
            _values[index] = value;
        }

        protected void removeKey(int index) throws AxionException {
            System.arraycopy(_keys, index + 1, _keys, index, _nodeSize - index - 1);
        }

        protected final void removeValue(int index) throws AxionException {
            System.arraycopy(_values, index + 1, _values, index, _nodeSize - index - 1);
        }

        protected void setKey(int index, Object key) throws AxionException {
            _keys[index] = key;
        }

        protected final void setValue(int index, int value) throws AxionException {
            _values[index] = value;
        }
    }

    public MemoryTTree(int maxNodeSize, Comparator comparator, TTreeMetaData metaData) throws AxionException {
        super(maxNodeSize, comparator, metaData);
    }

    public AbstractTTreeNode newNode() throws AxionException {
        return new TTreeNode(this);
    }

    public AbstractTTreeNode newNode(int fileId) throws AxionException {
        return new TTreeNode(this, fileId);
    }

}
