/*
 * $Id: TTreeIndexLoader.java,v 1.1 2005/06/30 01:14:44 ahimanikya Exp $
 * =======================================================================
 * Copyright (c) 2005 Axion Development Team.  All rights reserved.
 *  
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions 
 * are met:
 * 
 * 1. Redistributions of source code must retain the above 
 *    copyright notice, this list of conditions and the following 
 *    disclaimer. 
 *   
 * 2. Redistributions in binary form must reproduce the above copyright 
 *    notice, this list of conditions and the following disclaimer in 
 *    the documentation and/or other materials provided with the 
 *    distribution. 
 *   
 * 3. The names "Tigris", "Axion", nor the names of its contributors may 
 *    not be used to endorse or promote products derived from this 
 *    software without specific prior written permission. 
 *  
 * 4. Products derived from this software may not be called "Axion", nor 
 *    may "Tigris" or "Axion" appear in their names without specific prior
 *    written permission.
 *   
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY 
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * =======================================================================
 */

package org.axiondb.ext.indexes.ttree;

import java.io.File;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;

import org.axiondb.AxionException;
import org.axiondb.Column;
import org.axiondb.Index;
import org.axiondb.IndexLoader;
import org.axiondb.Table;
import org.axiondb.io.AxionFileSystem;
import org.axiondb.util.NullObject;

/**
 * Abstract IndexLoader for T-Tree.
 * 
 * @version $Revision: 1.1 $ $Date: 2005/06/30 01:14:44 $
 * @author Pavels Andrejevs
 */
public abstract class TTreeIndexLoader implements IndexLoader {
    protected static AxionFileSystem FS = new AxionFileSystem();

    public TTreeIndexLoader() {
    }

    // ---------------------------------------------------------------------

    public Object getNullKey() {
        return NullObject.INSTANCE;
    }

    public Index loadIndex(Table table, File dataDirectory) throws AxionException {
        ObjectInputStream in = null;
        try {
            String name = dataDirectory.getName();
            File file = new File(dataDirectory, name + ".DATA");
            in = FS.openObjectInputSteam(file);

            // read version number
            int ver = in.readInt();
            if (ver != 1) {
                throw new AxionException("Can't parse data file " + file + " for index " + name + ", unrecognized data file version " + ver);
            }

            String col = in.readUTF(); // read column name
            boolean unique = in.readBoolean(); // read unique flag
            int maxNodeSize = in.readInt(); // read minimization factor

            return makeIndex(name, table.getColumn(col), unique, dataDirectory, maxNodeSize, true);
        } catch (IOException e) {
            throw new AxionException(e);
        } finally {
            FS.closeInputStream(in);
        }
    }

    public void saveIndex(Index ndx, File dataDirectory) throws AxionException {
        DiskTTreeIndex index = (DiskTTreeIndex) ndx;
        ObjectOutputStream out = null;
        try {
            String name = index.getName();
            File file = new File(dataDirectory, name + ".DATA");
            out = FS.createObjectOutputSteam(file);

            out.writeInt(1); // write version number
            out.writeUTF(index.getIndexedColumn().getName()); // write column name
            out.writeBoolean(index.isUnique()); // write unique flag
            out.writeInt(index.getTTree().getMaxNodeSize()); // write max node size

            index.getTTree().save(dataDirectory);

        } catch (IOException e) {
            throw new AxionException(e);
        } finally {
            FS.closeOutputStream(out);
        }
    }

    public void saveIndexAfterTruncate(Index ndx, File dataDirectory) throws AxionException {
        DiskTTreeIndex index = (DiskTTreeIndex) ndx;
        index.getTTree().saveAfterTruncate(dataDirectory);
    }

    protected abstract Index makeIndex(String name, Column col, boolean unique, File dataDirectory, int maxNodeSize, boolean memorydb) throws AxionException;

}
