/*
 * $Id: PerformanceTester.java,v 1.1 2005/06/30 01:14:45 ahimanikya Exp $
 * ======================================================================= Copyright (c)
 * 2005 Axion Development Team. All rights reserved. Redistribution and use in source and
 * binary forms, with or without modification, are permitted provided that the following
 * conditions are met: 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer. 2. Redistributions in
 * binary form must reproduce the above copyright notice, this list of conditions and the
 * following disclaimer in the documentation and/or other materials provided with the
 * distribution. 3. The names "Tigris", "Axion", nor the names of its contributors may not
 * be used to endorse or promote products derived from this software without specific
 * prior written permission. 4. Products derived from this software may not be called
 * "Axion", nor may "Tigris" or "Axion" appear in their names without specific prior
 * written permission. THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
 * OF SUCH DAMAGE. =======================================================================
 */
package org.axiondb.ext.indexes.ttree;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;

public class PerformanceTester {

    private class Result {
        long empty;
        long popul;
        long query;
        long range;
    }

    public static void main(String[] args) throws Exception {
        String url = "jdbc:axiondb:egdb";
        if (args.length >= 1) {
            url += ":" + args[0];
        } 
        //else {
        //    url += ":testdb";
        //}
        int n = 40000; //99999;
        if (args.length >= 2) {
            n = Integer.parseInt(args[1]);
        }
        n += n * Math.random();
        System.out.println("Iterations: " + n);
        System.out.println("Connecting: " + url);
        new PerformanceTester(url, n);
    }

    private Result bb = new Result();
    private Result bu = new Result();
    private boolean cmp = true;
    private Connection conn;
    private final int NITERS;
    private boolean pre = true;
    private Result tt = new Result();
    private Result tu = new Result();

    public PerformanceTester(String uri, int n) throws Exception {
        NITERS = n;
        Class.forName("org.axiondb.jdbc.AxionDriver");
        conn = DriverManager.getConnection(uri);
        createTables();
        populTables();
        rangeTables();
        queryTables();
        emptyTables();

        double popul1 = ((double) bb.popul / tt.popul - 1.0) * 100;
        double range1 = ((double) bb.range / tt.range - 1.0) * 100;
        double query1 = ((double) bb.query / tt.query - 1.0) * 100;
        double empty1 = ((double) bb.empty / tt.empty - 1.0) * 100;
        System.out.println("Results TT/BB:");
        System.out.println("popul: " + popul1);
        System.out.println("range: " + range1);
        System.out.println("query: " + query1);
        System.out.println("empty: " + empty1);

        double popul2 = ((double) bu.popul / tu.popul - 1.0) * 100;
        double range2 = ((double) bu.range / tu.range - 1.0) * 100;
        double query2 = ((double) bu.query / tu.query - 1.0) * 100;
        double empty2 = ((double) bu.empty / tu.empty - 1.0) * 100;
        System.out.println("Results TU/BU:");
        System.out.println("popul: " + popul2);
        System.out.println("range: " + range2);
        System.out.println("query: " + query2);
        System.out.println("empty: " + empty2);

        System.out.println("Results AVG:");
        System.out.println("popul: " + (popul1 + popul2) / 2.0);
        System.out.println("range: " + (range1 + range2) / 2.0);
        System.out.println("query: " + (query1 + query2) / 2.0);
        System.out.println("empty: " + (empty1 + empty2) / 2.0);
    }

    public void createTables() throws Exception {
        System.out.println("Creating tables");
        Statement stmt = null;
        try {
            stmt = conn.createStatement();
            if (pre) {
                stmt.execute("create table b00  (id int, name varchar(100), " + "tag char, mod timestamp, doit boolean, sum real, "
                    + "val numeric(8));");
                stmt.execute("create table b00u (id int, name varchar(100), " + "tag char, mod timestamp, doit boolean, sum real, "
                    + "val numeric(8));");
                stmt.execute("create        btree index b00_id  on b00 (id)");
                stmt.execute("create unique btree index b00u_id on b00u(id)");

                stmt.execute("create table t00  (id int, name varchar(100), " + "tag char, mod timestamp, doit boolean, sum real, "
                    + "val numeric(8));");
                stmt.execute("create table t00u (id int, name varchar(100), " + "tag char, mod timestamp, doit boolean, sum real, "
                    + "val numeric(8));");
                stmt.execute("create        ttree index t00_id  on t00 (id)");
                stmt.execute("create unique ttree index t00u_id on t00u(id)");
            }
            if (cmp) {
                stmt.execute("create table bbb  (id int, name varchar(100), " + "tag char, mod timestamp, doit boolean, sum real, "
                    + "val numeric(8));");
                stmt.execute("create table bbbu (id int, name varchar(100), " + "tag char, mod timestamp, doit boolean, sum real, "
                    + "val numeric(8));");
                stmt.execute("create        btree index bbb_id  on bbb (id)");
                stmt.execute("create unique btree index bbbu_id on bbbu(id)");
            }
            stmt.execute("create table ttt  (id int, name varchar(100), " + "tag char, mod timestamp, doit boolean, sum real, " + "val numeric(8));");
            stmt.execute("create table tttu (id int, name varchar(100), " + "tag char, mod timestamp, doit boolean, sum real, " + "val numeric(8));");
            stmt.execute("create        ttree index ttt_id  on ttt (id);");
            stmt.execute("create unique ttree index tttu_id on tttu(id);");
        } finally {
            if (stmt != null)
                stmt.close();
        }
    }

    public long emptyTable(String table) throws Exception {
        long time;
        System.out.println("Emptying " + table);
        PreparedStatement stmt = null;
        boolean orgState = conn.getAutoCommit();
        conn.setAutoCommit(false);
        try {
            stmt = conn.prepareStatement("delete from " + table + " where id=?;");
            time = System.currentTimeMillis();
            for (int i = 0; i < NITERS; i++) {
                stmt.setInt(1, i);
                stmt.executeUpdate();
                commitIfNeeded(i);
            }
            time = System.currentTimeMillis() - time;
        } finally {
            if (stmt != null) {
                stmt.close();
            }
            conn.setAutoCommit(orgState);
        }
        System.out.println(time);
        return time;
    }

    public void emptyTables() throws Exception {
        if (pre) {
            emptyTable("b00");
            emptyTable("b00u");
            emptyTable("t00");
            emptyTable("t00u");
        }
        if (cmp) {
            bb.empty = emptyTable("bbb");
            bu.empty = emptyTable("bbbu");
        }
        tt.empty = emptyTable("ttt");
        tu.empty = emptyTable("tttu");
    }

    public long populTable(String table) throws Exception {
        long time;
        System.out.println("Populing " + table);
        PreparedStatement stmt = null;
        boolean orgState = conn.getAutoCommit();
        conn.setAutoCommit(false);
        try {
            stmt = conn.prepareStatement("insert into " + table + "(id, name) values(?, ?);");
            time = System.currentTimeMillis();
            for (int i = 0; i < NITERS; i++) {
                stmt.setInt(1, (i + 333) % NITERS);
                stmt.setString(2, "Jack" + i);
                stmt.executeUpdate();
                commitIfNeeded(i);
            }
            
            time = System.currentTimeMillis() - time;
        } finally {
            if (stmt != null) {
                stmt.close();
            }
            conn.setAutoCommit(orgState);
        }
        System.out.println(time);
        return time;
    }
    
    private void commitIfNeeded(int i) {
        if(i%1000 == 0) {
            try {
                conn.commit();
            } catch (SQLException e) {
                e.printStackTrace();
            }
        }
    }

    public void populTables() throws Exception {
        if (pre) {
            populTable("b00");
            populTable("b00u");
            populTable("t00");
            populTable("t00u");
        }
        if (cmp) {
            bb.popul = populTable("bbb");
            bu.popul = populTable("bbbu");
        }
        tt.popul = populTable("ttt");
        tu.popul = populTable("tttu");
    }

    public long queryTable(String table) throws Exception {
        long time;
        System.out.println("Querying " + table);
        PreparedStatement stmt = null;
        try {
            stmt = conn.prepareStatement("select id,name,tag,mod,doit,sum,val " + "from " + table + " where id=?");
            time = System.currentTimeMillis();
            for (int i = 0; i < NITERS; i++) {
                stmt.setInt(1, (i + 231) % NITERS);
                ResultSet rset = stmt.executeQuery();
                if (rset != null)
                    rset.close();
            }
            time = System.currentTimeMillis() - time;
        } finally {
            if (stmt != null)
                stmt.close();
        }
        System.out.println(time);
        return time;
    }

    public void queryTables() throws Exception {
        if (pre) {
            queryTable("b00");
            queryTable("b00u");
            queryTable("t00");
            queryTable("t00u");
        }
        if (cmp) {
            bb.query = queryTable("bbb");
            bu.query = queryTable("bbbu");
        }
        tt.query = queryTable("ttt");
        tu.query = queryTable("tttu");
    }

    public long rangeTable(String table) throws Exception {
        long time;
        System.out.println("Rangeing " + table);
        String query = "select id,name,tag,mod,doit,sum,val from " + table + " where id>111";
        Statement stmt = null;
        ResultSet rset = null;
        try {
            stmt = conn.createStatement();
            time = System.currentTimeMillis();
            rset = stmt.executeQuery(query);
            time = System.currentTimeMillis() - time;
        } finally {
            if (rset != null)
                rset.close();
            if (stmt != null)
                stmt.close();
        }
        System.out.println(time);
        return time;
    }

    public void rangeTables() throws Exception {
        if (pre) {
            rangeTable("b00");
            rangeTable("b00u");
            rangeTable("t00");
            rangeTable("t00u");
        }
        if (cmp) {
            bb.range = rangeTable("bbb");
            bu.range = rangeTable("bbbu");
        }
        tt.range = rangeTable("ttt");
        tu.range = rangeTable("tttu");
    }
}
