/*
 * $Id: OrderNode.java,v 1.6 2004/08/27 03:21:20 ahimanikya Exp $
 * =======================================================================
 * Copyright (c) 2002-2004 Axion Development Team.  All rights reserved.
 *  
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions 
 * are met:
 * 
 * 1. Redistributions of source code must retain the above 
 *    copyright notice, this list of conditions and the following 
 *    disclaimer. 
 *   
 * 2. Redistributions in binary form must reproduce the above copyright 
 *    notice, this list of conditions and the following disclaimer in 
 *    the documentation and/or other materials provided with the 
 *    distribution. 
 *   
 * 3. The names "Tigris", "Axion", nor the names of its contributors may 
 *    not be used to endorse or promote products derived from this 
 *    software without specific prior written permission. 
 *  
 * 4. Products derived from this software may not be called "Axion", nor 
 *    may "Tigris" or "Axion" appear in their names without specific prior
 *    written permission.
 *   
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY 
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * =======================================================================
 */

package org.axiondb;

/**
 * One part of an ORDER BY clause.
 * 
 * @version $Revision: 1.6 $ $Date: 2004/08/27 03:21:20 $
 * @author Morgan Delagrange
 * @author Rodney Waldhoff
 */
public class OrderNode {
    /**
     * Create an <code>OrderNode</code> that sorts the given <i>selectable </i> in
     * ascending or descending order.
     * 
     * @param selectable the {@link Selectable}to sort by
     * @param descending when <code>true</code> sort in descending (greatest to least)
     *        order
     */
    public OrderNode(Selectable selectable, boolean descending) {
        setSelectable(selectable);
        setDescending(descending);
    }

    public Selectable getSelectable() {
        return _sel;
    }

    public boolean isDescending() {
        return _descending;
    }

    public void setDescending(boolean desc) {
        _descending = desc;
    }

    public void setSelectable(Selectable sel) {
        _sel = sel;
    }

    public String toString() {
        return getSelectable() + (isDescending() ? " desc" : " asc");
    }

    private boolean _descending = false;
    private Selectable _sel = null;
}