/*
 * $Id: BaseIndexFactory.java,v 1.6 2005/06/29 21:09:30 ahimanikya Exp $
 * =======================================================================
 * Copyright (c) 2004-2005 Axion Development Team.  All rights reserved.
 *  
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions 
 * are met:
 * 
 * 1. Redistributions of source code must retain the above 
 *    copyright notice, this list of conditions and the following 
 *    disclaimer. 
 *   
 * 2. Redistributions in binary form must reproduce the above copyright 
 *    notice, this list of conditions and the following disclaimer in 
 *    the documentation and/or other materials provided with the 
 *    distribution. 
 *   
 * 3. The names "Tigris", "Axion", nor the names of its contributors may 
 *    not be used to endorse or promote products derived from this 
 *    software without specific prior written permission. 
 *  
 * 4. Products derived from this software may not be called "Axion", nor 
 *    may "Tigris" or "Axion" appear in their names without specific prior
 *    written permission.
 *   
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY 
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * =======================================================================
 */
package org.axiondb.engine.indexes;

import org.axiondb.AxionException;
import org.axiondb.Column;
import org.axiondb.Index;
import org.axiondb.IndexFactory;
import org.axiondb.Table;
import org.axiondb.util.StringIdentifierGenerator;

/**
 * Abstract base class for {@link IndexFactory}to hold common methods.
 * 
 * @version $Revision: 1.6 $ $Date: 2005/06/29 21:09:30 $
 * @author Jonathan Giron
 */
public abstract class BaseIndexFactory implements IndexFactory {

    protected BaseIndexFactory() {
    }

    /**
     * Creates a new system instance with an autogenerated index name that combines the
     * table name, column name, and current UNIX time (expressed in hexadecimal).
     * 
     * @see org.axiondb.IndexFactory.makeNewSystemInstance(org.axiondb.Table,
     *      org.axiondb.Column, boolean)
     */
    public synchronized Index makeNewSystemInstance(Table table, Column col, boolean memorydb) throws AxionException {
        boolean unique = table.isPrimaryKeyConstraintExists(col.getName()) || table.isUniqueConstraintExists(col.getName());
        return makeNewInstance(StringIdentifierGenerator.INSTANCE.next16DigitIdentifier("SYS"), col, unique, memorydb);
    }
}