/*
 * $Id: ForwardOnlyResultSet.java,v 1.3 2007/11/13 19:04:01 rwald Exp $
 * =======================================================================
 * Copyright (c) 2005 Axion Development Team.  All rights reserved.
 *  
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions 
 * are met:
 * 
 * 1. Redistributions of source code must retain the above 
 *    copyright notice, this list of conditions and the following 
 *    disclaimer. 
 *   
 * 2. Redistributions in binary form must reproduce the above copyright 
 *    notice, this list of conditions and the following disclaimer in 
 *    the documentation and/or other materials provided with the 
 *    distribution. 
 *   
 * 3. The names "Tigris", "Axion", nor the names of its contributors may 
 *    not be used to endorse or promote products derived from this 
 *    software without specific prior written permission. 
 *  
 * 4. Products derived from this software may not be called "Axion", nor 
 *    may "Tigris" or "Axion" appear in their names without specific prior
 *    written permission.
 *   
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY 
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * =======================================================================
 */
package org.axiondb.jdbc;

import java.io.InputStream;
import java.io.Reader;
import java.sql.NClob;
import java.sql.ResultSet;
import java.sql.RowId;
import java.sql.SQLException;
import java.sql.SQLXML;
import java.util.Map;

/**
 *
 * @author Jonathan Giron
 * @version $Revision: 1.3 $
 */
public final class ForwardOnlyResultSet extends BaseAxionResultSetDecorator {

    /**
     * @param rs ResultSet that needs to be decorated
     */
    public ForwardOnlyResultSet(ResultSet rs) {
        super(rs);
    }

    public boolean absolute(int row) throws SQLException {
        throw new SQLException("Forward-only ResultSet - not supported.");
    }

    public void afterLast() throws SQLException {
        throw new SQLException("Forward-only ResultSet - not supported.");
    }

    public void beforeFirst() throws SQLException {
        throw new SQLException("Forward-only ResultSet - not supported.");
    }

    public boolean first() throws SQLException {
        throw new SQLException("Forward-only ResultSet - not supported.");
    }

    public boolean last() throws SQLException {
        throw new SQLException("Forward-only ResultSet - not supported.");
    }

    public boolean previous() throws SQLException {
        throw new SQLException("Forward-only ResultSet - not supported.");
    }

    public boolean relative(int rows) throws SQLException {
        if (rows < 0) {
            throw new SQLException("Forward-only ResultSet - not supported.");
        }

        return super.relative(rows);
    }

    public void setFetchDirection(int direction) throws SQLException {
        if (ResultSet.FETCH_FORWARD == direction) {
            super.setFetchDirection(direction);
        } else {
            throw new SQLException("Forward-only ResultSet - not supported.");
        }
    }

	@Override
	public int getHoldability() throws SQLException {
		throw new SQLException("Not supported");
	}

	@Override
	public Reader getNCharacterStream(int arg0) throws SQLException {
		throw new SQLException("Not supported");
	}

	@Override
	public Reader getNCharacterStream(String arg0) throws SQLException {
		throw new SQLException("Not supported");
	}

	@Override
	public NClob getNClob(int arg0) throws SQLException {
		throw new SQLException("Not supported");
	}

	@Override
	public NClob getNClob(String arg0) throws SQLException {
		throw new SQLException("Not supported");
	}

	@Override
	public String getNString(int arg0) throws SQLException {
		throw new SQLException("Not supported");
	}

	@Override
	public String getNString(String arg0) throws SQLException {
		throw new SQLException("Not supported");
	}

	@Override
	public RowId getRowId(int arg0) throws SQLException {
		throw new SQLException("Not supported");
	}

	@Override
	public RowId getRowId(String arg0) throws SQLException {
		throw new SQLException("Not supported");
	}

	@Override
	public SQLXML getSQLXML(int arg0) throws SQLException {
		throw new SQLException("Not supported");
	}

	@Override
	public SQLXML getSQLXML(String arg0) throws SQLException {
		throw new SQLException("Not supported");
	}

	@Override
	public boolean isClosed() throws SQLException {
		throw new SQLException("Not supported");
	}

	@Override
	public void updateAsciiStream(int arg0, InputStream arg1)
			throws SQLException {
		throw new SQLException("Not supported");
	}

	@Override
	public void updateAsciiStream(String arg0, InputStream arg1)
			throws SQLException {
		throw new SQLException("Not supported");
	}

	@Override
	public void updateAsciiStream(int arg0, InputStream arg1, long arg2)
			throws SQLException {
		throw new SQLException("Not supported");
	}

	@Override
	public void updateAsciiStream(String arg0, InputStream arg1, long arg2)
			throws SQLException {
		throw new SQLException("Not supported");
	}

	@Override
	public void updateBinaryStream(int arg0, InputStream arg1)
			throws SQLException {
		throw new SQLException("Not supported");
	}

	@Override
	public void updateBinaryStream(String arg0, InputStream arg1)
			throws SQLException {
		throw new SQLException("Not supported");
	}

	@Override
	public void updateBinaryStream(int arg0, InputStream arg1, long arg2)
			throws SQLException {
		throw new SQLException("Not supported");
	}

	@Override
	public void updateBinaryStream(String arg0, InputStream arg1, long arg2)
			throws SQLException {
		throw new SQLException("Not supported");
	}

	@Override
	public void updateBlob(int arg0, InputStream arg1) throws SQLException {
		throw new SQLException("Not supported");
	}

	@Override
	public void updateBlob(String arg0, InputStream arg1) throws SQLException {
		throw new SQLException("Not supported");
	}

	@Override
	public void updateBlob(int arg0, InputStream arg1, long arg2)
			throws SQLException {
		throw new SQLException("Not supported");
	}

	@Override
	public void updateBlob(String arg0, InputStream arg1, long arg2)
			throws SQLException {
		throw new SQLException("Not supported");
	}

	@Override
	public void updateCharacterStream(int arg0, Reader arg1)
			throws SQLException {
		throw new SQLException("Not supported");
	}

	@Override
	public void updateCharacterStream(String arg0, Reader arg1)
			throws SQLException {
		throw new SQLException("Not supported");
	}

	@Override
	public void updateCharacterStream(int arg0, Reader arg1, long arg2)
			throws SQLException {
		throw new SQLException("Not supported");
	}

	@Override
	public void updateCharacterStream(String arg0, Reader arg1, long arg2)
			throws SQLException {
		throw new SQLException("Not supported");
	}

	@Override
	public void updateClob(int arg0, Reader arg1) throws SQLException {
		throw new SQLException("Not supported");
	}

	@Override
	public void updateClob(String arg0, Reader arg1) throws SQLException {
		throw new SQLException("Not supported");
	}

	@Override
	public void updateClob(int arg0, Reader arg1, long arg2)
			throws SQLException {
		throw new SQLException("Not supported");
	}

	@Override
	public void updateClob(String arg0, Reader arg1, long arg2)
			throws SQLException {
		throw new SQLException("Not supported");
	}

	@Override
	public void updateNCharacterStream(int arg0, Reader arg1)
			throws SQLException {
		throw new SQLException("Not supported");
	}

	@Override
	public void updateNCharacterStream(String arg0, Reader arg1)
			throws SQLException {
		throw new SQLException("Not supported");
	}

	@Override
	public void updateNCharacterStream(int arg0, Reader arg1, long arg2)
			throws SQLException {
		throw new SQLException("Not supported");
	}

	@Override
	public void updateNCharacterStream(String arg0, Reader arg1, long arg2)
			throws SQLException {
		throw new SQLException("Not supported");
	}

	@Override
	public void updateNClob(int arg0, NClob arg1) throws SQLException {
		throw new SQLException("Not supported");
	}

	@Override
	public void updateNClob(String arg0, NClob arg1) throws SQLException {
		throw new SQLException("Not supported");
	}

	@Override
	public void updateNClob(int arg0, Reader arg1) throws SQLException {
		throw new SQLException("Not supported");
	}

	@Override
	public void updateNClob(String arg0, Reader arg1) throws SQLException {
		throw new SQLException("Not supported");
	}

	@Override
	public void updateNClob(int arg0, Reader arg1, long arg2)
			throws SQLException {
		throw new SQLException("Not supported");
	}

	@Override
	public void updateNClob(String arg0, Reader arg1, long arg2)
			throws SQLException {
		throw new SQLException("Not supported");
	}

	@Override
	public void updateNString(int arg0, String arg1) throws SQLException {
		throw new SQLException("Not supported");
	}

	@Override
	public void updateNString(String arg0, String arg1) throws SQLException {
		throw new SQLException("Not supported");
	}

	@Override
	public void updateRowId(int arg0, RowId arg1) throws SQLException {
		throw new SQLException("Not supported");
	}

	@Override
	public void updateRowId(String arg0, RowId arg1) throws SQLException {
		throw new SQLException("Not supported");
	}

	@Override
	public void updateSQLXML(int arg0, SQLXML arg1) throws SQLException {
		throw new SQLException("Not supported");
	}

	@Override
	public void updateSQLXML(String arg0, SQLXML arg1) throws SQLException {
		throw new SQLException("Not supported");
	}

	@Override
	public boolean isWrapperFor(Class<?> arg0) throws SQLException {
		throw new SQLException("Not supported");
	}

	@Override
	public <T> T unwrap(Class<T> arg0) throws SQLException {
		throw new SQLException("Not supported");
	}
}