/*
 * $Id: TestAll.java,v 1.28 2005/04/24 21:35:59 ahimanikya Exp $
 * =======================================================================
 * Copyright (c) 2002-2003 Axion Development Team.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The names "Tigris", "Axion", nor the names of its contributors may
 *    not be used to endorse or promote products derived from this
 *    software without specific prior written permission.
 *
 * 4. Products derived from this software may not be called "Axion", nor
 *    may "Tigris" or "Axion" appear in their names without specific prior
 *    written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * =======================================================================
 */

package org.axiondb;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

/**
 * Root test suite.
 *
 * @version $Revision: 1.28 $ $Date: 2005/04/24 21:35:59 $
 * @author Rodney Waldhoff
 * @author Chuck Burdick
 */
public class TestAll extends TestCase {

    public TestAll(String testName) {
        super(testName);
    }

    public static void main(String args[]) {
        String[] testCaseName = { TestAll.class.getName() };
        junit.textui.TestRunner.main(testCaseName);
    }

    public static Test suite() {
        TestSuite suite = new TestSuite();
        suite.addTest(TestTableIdentifier.suite());
        suite.addTest(TestColumnIdentifier.suite());
        suite.addTest(TestColumn.suite());
        suite.addTest(TestLiteral.suite());
        suite.addTest(TestSequence.suite());
        suite.addTest(TestPersistentDatabase.suite());
        suite.addTest(TestFunctional.suite());
        suite.addTest(TestOrderNode.suite());
        suite.addTest(TestBindVariable.suite());
        suite.addTest(TestBaseSelectable.suite());
        suite.addTest(TestSequenceEvaluator.suite());
        suite.addTest(TestRowComparator.suite());
        
        suite.addTest(org.axiondb.util.TestAll.suite());

        suite.addTest(org.axiondb.constraints.TestAll.suite());
        suite.addTest(org.axiondb.parser.TestAll.suite());
        suite.addTest(org.axiondb.types.TestAll.suite());
        suite.addTest(org.axiondb.jdbc.TestAll.suite());
        suite.addTest(org.axiondb.functions.TestAll.suite());
        suite.addTest(org.axiondb.functional.TestAll.suite());
        
        suite.addTest(org.axiondb.engine.TestAll.suite());
        suite.addTest(org.axiondb.engine.commands.TestAll.suite());
        suite.addTest(org.axiondb.engine.indexes.TestAll.suite());
        suite.addTest(org.axiondb.engine.rowiterators.TestAll.suite());
        suite.addTest(org.axiondb.engine.rows.TestAll.suite());
        suite.addTest(org.axiondb.engine.tables.TestAll.suite());
        suite.addTest(org.axiondb.engine.visitors.TestAll.suite());
        suite.addTest(org.axiondb.event.TestAll.suite());
        
        // BTree tests execute very slowly if we run the util suite at this point - moved to
        // top of order.  Perhaps this is caused by memory/gc issues?
        // suite.addTest(org.axiondb.util.TestAll.suite());
        
        suite.addTest(org.axiondb.tools.TestAll.suite());
        suite.addTest(TestAxionException.suite()); 

        return suite;
    }
}

