/*
 * $Id: TestAndFunction.java,v 1.3 2005/06/18 01:03:44 ahimanikya Exp $
 * =======================================================================
 * Copyright (c) 2003-2005 Axion Development Team.  All rights reserved.
 *  
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions 
 * are met:
 * 
 * 1. Redistributions of source code must retain the above 
 *    copyright notice, this list of conditions and the following 
 *    disclaimer. 
 *   
 * 2. Redistributions in binary form must reproduce the above copyright 
 *    notice, this list of conditions and the following disclaimer in 
 *    the documentation and/or other materials provided with the 
 *    distribution. 
 *   
 * 3. The names "Tigris", "Axion", nor the names of its contributors may 
 *    not be used to endorse or promote products derived from this 
 *    software without specific prior written permission. 
 *  
 * 4. Products derived from this software may not be called "Axion", nor 
 *    may "Tigris" or "Axion" appear in their names without specific prior
 *    written permission.
 *   
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY 
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * =======================================================================
 */

package org.axiondb.functions;

import junit.framework.Test;
import junit.framework.TestSuite;

/**
 * @version $Revision: 1.3 $ $Date: 2005/06/18 01:03:44 $
 * @author Rodney Waldhoff
 * @author Jonathan Giron
 */
public class TestAndFunction extends BaseBooleanBranchFunctionTest {

    //------------------------------------------------------------ Conventional

    public TestAndFunction(String testName) {
        super(testName);
    }

    public static Test suite() {
        TestSuite suite = new TestSuite(TestAndFunction.class);
        return suite;
    }

    //--------------------------------------------------------------- Lifecycle

    //--------------------------------------------------------------- Framework
    
    protected ConcreteFunction makeFunction() {        
        return new AndFunction();
    }

    protected Boolean evaluate(Boolean[] values) {
        for(int i=0;i<values.length;i++) {
            if(!values[i].booleanValue()) {
                return Boolean.FALSE;           
            }
        }
        return Boolean.TRUE;
    }
    
    /*
     * Defined in Table 11, ISO/IEC 9075-2:2003, Section 6.34.  
     * @see org.axiondb.functions.BaseBooleanBranchFunctionTest#getTruthTable()
     */
    protected Object[][] getTruthTable() {
        return new Object[][] {
            new Object[] { Boolean.TRUE, Boolean.TRUE, Boolean.TRUE },
            new Object[] { Boolean.TRUE, Boolean.FALSE, Boolean.FALSE },
            new Object[] { Boolean.TRUE, null, null },
            new Object[] { Boolean.FALSE, Boolean.TRUE, Boolean.FALSE },
            new Object[] { Boolean.FALSE, Boolean.FALSE, Boolean.FALSE },
            new Object[] { Boolean.FALSE, null, Boolean.FALSE },
            new Object[] { null, Boolean.TRUE, null},
            new Object[] { null, Boolean.FALSE, Boolean.FALSE },
            new Object[] { null, null, null}
        };
    }

    protected String getExpressionDisplay(Object[] arguments) {
        StringBuffer buf = new StringBuffer(25);
        for (int i = 0; i < arguments.length; i++) {
            if (i != 0) { 
                buf.append(" AND ");
            }
            buf.append(arguments[i]);
        }
        return buf.toString();
    }
    
    //------------------------------------------------------------------- Tests

    public void testMakeNewInstance() {
        AndFunction function = new AndFunction();
        assertTrue(function.makeNewInstance() instanceof AndFunction);
        assertTrue(function.makeNewInstance() != function.makeNewInstance());
    }
}
