/*
 * $Id: BlobInserts.java,v 1.4 2005/01/27 23:32:02 rwald Exp $
 * =======================================================================
 * Copyright (c) 2005 Axion Development Team.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The names "Tigris", "Axion", nor the names of its contributors may
 *    not be used to endorse or promote products derived from this
 *    software without specific prior written permission.
 *
 * 4. Products derived from this software may not be called "Axion", nor
 *    may "Tigris" or "Axion" appear in their names without specific prior
 *    written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * =======================================================================
 */
package org.axiondb.profile;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Random;

import org.axiondb.types.ByteArrayBlob;


/**
 * Derived from Chas Emerick's 25-Jan-2005 post to the user's list.
 * @author Chas Emerick
 * @author Rodney Waldhoff
 */
public class BlobInserts {
    private static Random _random = new Random();

    static {
        try {
            Class.forName("org.axiondb.jdbc.AxionDriver");
        } catch(Exception e) {
            e.printStackTrace();
        }
    }
    
    public static void main(String[] args) throws Exception {
        String connectString = "jdbc:axiondb:test:/tmp/testdb" +_random.nextLong();        
        int numLoops = 10000;
        int reportEvery = 1000;
        boolean commitEveryBatch = true;
        boolean useConstraint = false;
        
        if(args.length > 0 && "--help".equals(args[0])) {
            System.out.println("args: <total # of inserts> <batch size> <commit after every batch> <use constraint> <connect string>");
            System.out.println("defaults: " + numLoops + " "  + reportEvery + " " + commitEveryBatch + " " + useConstraint + " " + connectString);
            return;
        }

        switch(args.length) {
            case 5:
                connectString = args[4];
            case 4:
                useConstraint = Boolean.valueOf(args[3]).booleanValue();
            case 3:
                commitEveryBatch = Boolean.valueOf(args[2]).booleanValue();
            case 2:
                reportEvery = Integer.parseInt(args[1]);
            case 1:
                numLoops = Integer.parseInt(args[0]);
                break;
        }

        System.out.println("Total number of inserts: " + numLoops);
        System.out.println("Reporting every: " + reportEvery);
        System.out.println("Commit after each batch? " + commitEveryBatch);
        System.out.println("Apply constraint? " + useConstraint);
        System.out.println("Connect String: " + connectString);

        System.out.println("Running...");
        System.out.println("");
        
        Connection conn = DriverManager.getConnection(connectString);
        conn.setAutoCommit(false);
        run(conn,numLoops,reportEvery,commitEveryBatch,useConstraint);
        conn.close();
    }

    private static void run(Connection conn, int loops, int reportEvery, boolean commitEveryBatch, boolean useConstraint) throws SQLException {
        byte[] data = new byte[25 * 1024];
        _random.nextBytes(data);
        {
            Statement stmt = conn.createStatement();
            stmt.executeUpdate("create table data_test (id int " + (useConstraint ? "PRIMARY KEY " : "") + ", type_cd varchar(5), filedata blob);");
            stmt.close();
            conn.commit();
        }
        PreparedStatement pstmt = conn.prepareStatement("insert into data_test (id, type_cd, filedata) values (?, ?, ?)");
        long start = System.currentTimeMillis();
        long time = start;
        for (int i = 1; i <= loops; i++) {
           pstmt.setInt(1, i);
           pstmt.setString(2, "t_ind");
           pstmt.setBlob(3, new ByteArrayBlob(data));
           pstmt.executeUpdate();
           if (i % reportEvery == 0) {
               if(commitEveryBatch) { conn.commit(); }
               long stop = System.currentTimeMillis();
               System.out.println("Batch of " + reportEvery + " inserts took " + (stop - time) + " ms [" + (((double)stop-time)/reportEvery) + " ms/insert]");
               time = System.currentTimeMillis();
           }
        }
        conn.commit();
        long stop = System.currentTimeMillis();
        System.out.println("Total of " + loops + " inserts took " + (stop - start) + " ms [" + (((double)stop-start)/loops) + " ms/insert]");
    }
    
}
